{
	* This program written by code_glitch AKA Benjamin Gregg-Smith as a fast demonstration
		of Prometheus' video and core units 09 March 2011
	* This is free code, and serves as a tutorial for prometheus' of sorts and also basic documentation.
	* This designed on Prometheus with all latest revisions (SVN revision 51/52) it may not work on some other revisions
	* Please do not claim this code as your own, but you don't have to say where you got it from either Ie.I don't ask for referals
	* If you use this or any of my code for commercial purposes please do let me know at code_glitch@live.co.uk or on pascalgamedevelopment.com
	* This is all compiled with FPC 2.2.4 and tested on Ubuntu 10.10 x64
	* There are ~135 lines of code without comments
	* I use the math unit as it has not yet been integrated to Prometheus_Core. It may find implementation there some time though.
	* Thank you for reading, enjoy.
}

program ShipDemo;

uses
	Prometheus_Vid,
	Prometheus_Core,
	Math;

type
	tEnvironment = Object
			Drag: Real;
			GravityX, GravityY: Real;
			
			procedure Initialize();
		end;
	tPlayer = Object
			Ship: Image;
			Bearing: Int64;
			Speed: Real;
			PositionX, PositionY: Real;
			
			procedure Initialize();
			procedure Update();
		end;
	tApplication = Object
			OSDelay: Int64;
			
			procedure Initialize();
			
			procedure HandleInput();
			procedure Draw();
			procedure Wait();
		end;

var
	Environment: tEnvironment;
	Application: tApplication;
	Player: tPlayer;

procedure tEnvironment.Initialize();
begin
	//A value such as +67 will make it 67 pixels faster per cycle
	//A value such as -5.673 will make things 5.673 pixels slower
	
	//Because we don't limit player velocity, we could end up going backwards.
	//	Just like when you don't swim in an omni-directional stream...
	Drag := 0;
	
	//Give each dimension some gravity.
	GravityX := 0;
	GravityY := 0;
end;

procedure tPlayer.Update();
begin
	Speed := Speed + Environment.Drag; //Apply the drag we set earlier
	PositionX := PositionX + ( Speed * Sin(degtorad(Bearing)) ); //Use the bearing and speed to work out what change
	PositionY := PositionY - ( Speed * Cos(degtorad(Bearing)) ); //	we should apply to the X and Y of the ship
	
	PositionX := PositionX + Environment.GravityX; //Apply gravity.
	PositionY := PositionY + Environment.GravityY;
	
	if PositionX + (Ship.Width div 2) >= GetScreenWidth then //Prevent our ship going off-screen
		PositionX := GetScreenWidth - (Ship.Width div 2);
	if PositionX - (Ship.Width div 2) <= 0 then
		PositionX := Ship.Width div 2;
	if PositionY + (Ship.Width div 2) >= GetScreenHeight then
		PositionY := GetScreenHeight - (Ship.Width div 2);
	if PositionY - (Ship.Width div 2) <= 0 then
		PositionY := Ship.Width div 2;
end;

procedure tPlayer.Initialize();
begin
	Ship.Load('PlayerShip.png'); //Load the ship image from disk
	
	PositionX := 200; //Starting Positions
	PositionY := 200;
end;

procedure tApplication.Initialize();
begin
	PrometheusVid_Start(); //Get the video going
	PrometheusCore_Start(); //Event handling
	
	CreateWindow(800, 600, 32); //Make a window 800px wide by 600px high in 32 bit colour
	UpdateWindow(); //Apply changes to our window just in case something went wrong the first time. Good practice but usually not required.
	
	OSDelay := 25; //1000ms / 25 = ~40 f/sec (give the operating system some time to think)
	
	Environment.Initialize(); //Set the gravity drag and etc...
	Player.Initialize(); //Let use set defaults for the player
end;

procedure tApplication.HandleInput();
begin
	if IsKeyDown then //Is a key pressed?
		begin
			if GetKeyDown = 273 then //Up Arrow
				begin
					Player.Speed := Player.Speed + 1;
				end;
			if GetKeyDown = 274 then //Down Arrow
				begin
					Player.Speed := Player.Speed - 1;
				end;
			if GetKeyDown = 275 then //Right Arrow
				begin
					Player.Bearing := Player.Bearing + 1;
				end;
			if GetKeyDown = 276 then //Left Arrow
				begin
					Player.Bearing := Player.Bearing - 1;
				end;
		end;
end;

procedure tApplication.Draw();
begin
	ClearCanvas(); //Clear the screen
	
	{
		* This draws to the CENTRE co-ordinate of the image because we loaded prometheus with all default values.
			* From r53 or so, use PrometheusDrawMode(Mode) to change this. Options are:
			* 	UpperLeft / TopLeft
			* 	UpperRight / TopRight
			* 	LowerLeft / BottomLeft
			* 	LowerRight / BottomRight
			* 	Centre / Middle
			* So you could either do PrometheusDrawMode(UpperLeft), PrometheusDrawMode(TopLeft) and etc...
			* This mode applies to ALL images.
		* We pass DrawRotated(), the X position, Y Position, and Angle of rotation in DEGREES and CLOCKWISE!
	}
	Player.Ship.DrawRotated( trunc(Player.PositionX), trunc(Player.PositionY), trunc(Player.Bearing) );
	
	{
		* Alternatively you could do:
			* Player.Ship.Rotate( trunc(Bearing) );
			* Player.Ship.Draw( trunc(Player.PositionX), trunc(Player.PositionY) );
	}
	
	{
		* Now we simply draw it all to the screen by updating the screen to video buffer
	}
	UpdateCanvas();
end;

procedure tApplication.Wait();
begin
	{
		* Prometheus procedure equal to crt's delay(Time) and sdl's Sdl_Delay(Time) if you dont want to include the sdl unit.
	}
	Pause(OSDelay);
end;

begin
	//Load evedrything up
	Application.Initialize();
	repeat
		{
			* GetInput from Prometheus_Core updates Prometheus' information on current events which can later be accessed with
				* IsKeyDown
				* GetKeyDown
				* IsMouse
				* IsMouseDown
				* And etc
		}
		GetInput();
		
		//If certain events happen, lets do something in response
		Application.HandleInput();
		
		//Update player co-ordinates and etc
		Player.Update();
		
		//Draw everything onto the screen
		Application.Draw();
		
		//Give the OS some time to think and not hog the CPU
		Application.Wait();
		until IsClose();
end.
